@@ -26,6 +26,7 @@
 #include <KPluginFactory>
 #include <KProcess>
 #include <KStandardDirs>
+#include <KMessageBox>
 #include <QtGui>
 #include <QX11EmbedContainer>
 #include "ui_gui.h"
@@ -56,7 +57,7 @@ GTKConfigKCModule::GTKConfigKCModule(QWidget* parent, const QVariantList& args )
     , m_saveEnabled(true)
 {
     Q_UNUSED(args);
-    KAboutData *acercade = new KAboutData("cgc","kcm_cgc",ki18n("KDE GTK Config"), "2.0",
+    KAboutData *acercade = new KAboutData("cgc","kcm_cgc",ki18n("KDE GTK Config"), "2.2",
                     ki18n("Configure your GTK Applications"),
                     KAboutData::License_LGPL_V3,
                     ki18n("Copyright 2011 José Antonio Sánchez Reynaga"));
@@ -75,16 +76,29 @@ GTKConfigKCModule::GTKConfigKCModule(QWidget* parent, const QVariantList& args )
     m_tempGtk2Preview = KGlobal::dirs()->saveLocation("tmp", "gtkrc-2.0", false);
     m_tempGtk3Preview = KGlobal::dirs()->saveLocation("tmp", ".config/gtk-3.0/settings.ini", false);
     
-    ui->gtk2Preview->setIcon(KIcon("document-preview")); //!! for some reason it doesn't work with QIcon::fromTheme
-    ui->gtk3Preview->setIcon(KIcon("document-preview"));
+    const KIcon previewIcon("document-preview");
+    ui->gtk2Preview->setIcon(previewIcon);
+    ui->gtk3Preview->setIcon(previewIcon);
+    
+    QString gtk2Preview = KStandardDirs::findExe("gtk_preview");
+    QString gtk3Preview = KStandardDirs::findExe("gtk3_preview");
     
     m_p2 = new KProcess(this);
     m_p2->setEnv("GTK2_RC_FILES", m_tempGtk2Preview, true);
-    *m_p2 << KStandardDirs::findExe("gtk_preview");
+    if(!gtk2Preview.isEmpty()) {
+        *m_p2 << gtk2Preview;
+        connect(m_p2, SIGNAL(finished(int)), this, SLOT(untogglePreview()));
+    }
     
     m_p3 = new KProcess(this);
     m_p3->setEnv("XDG_CONFIG_HOME", KGlobal::dirs()->saveLocation("tmp", ".config"));
-    *m_p3 << KStandardDirs::findExe("gtk3_preview");
+    if(gtk3Preview.isEmpty()) {
+        *m_p3 << gtk3Preview;
+        connect(m_p3, SIGNAL(finished(int)), this, SLOT(untogglePreview()));
+    }
+    
+    ui->gtk2Preview->setVisible(!gtk2Preview.isEmpty());
+    ui->gtk3Preview->setVisible(!gtk3Preview.isEmpty());
     
     //UI changes
     connect(ui->cb_theme, SIGNAL(currentIndexChanged(int)), this, SLOT(appChanged()));
@@ -102,9 +116,6 @@ GTKConfigKCModule::GTKConfigKCModule(QWidget* parent, const QVariantList& args )
     connect(ui->gtk2Preview, SIGNAL(clicked(bool)), this, SLOT(runGtk2IfNecessary(bool)));
     connect(ui->gtk3Preview, SIGNAL(clicked(bool)), this, SLOT(runGtk3IfNecessary(bool)));
     
-    connect(m_p2, SIGNAL(finished(int)), this, SLOT(untogglePreview()));
-    connect(m_p3, SIGNAL(finished(int)), this, SLOT(untogglePreview()));
-    
     QMenu* m = new QMenu(this);
     m->addAction(KIcon("get-hot-new-stuff"), i18n("Download GTK2 themes..."), this, SLOT(showThemeGHNS()));
     m->addAction(KIcon("get-hot-new-stuff"), i18n("Download GTK3 themes..."), this, SLOT(installThemeGTK3GHNS()));
@@ -130,14 +141,7 @@ GTKConfigKCModule::~GTKConfigKCModule()
 
 QString fontToString(const QFont& f)
 {
-    QString style;
-
-    if(f.bold())
-        style += " bold";
-    if(f.italic())
-        style += " italic";
-    
-    return f.family() + style + ' ' + QString::number(f.pointSize());
+    return f.family() + ' ' + f.styleName() + ' ' + QString::number(f.pointSize());
 }
 
 void GTKConfigKCModule::syncUI()
@@ -171,15 +175,23 @@ void GTKConfigKCModule::installThemeGTK3GHNS()
 
 QFont stringToFont(const QString& font)
 {
-    QRegExp fontRx(QString(" (italic)? *(bold)? *([0-9]+)$"));
-    int pos = fontRx.indexIn(font);
-    QString fontFamily = font.left(pos);
+    QFontDatabase fdb;
+    QString fontFamily;
+    int familyIdx=-1;
+    for(int idx=font.indexOf(' '); idx<font.size(); idx=font.indexOf(' ', idx+1)) {
+        QString testFont = font.left(idx);
+        if(!fdb.hasFamily(testFont))
+            break;
+        fontFamily = testFont;
+        familyIdx = idx;
+    }
+    QRegExp fontRx(QString("( [a-zA-Z ]*) +([0-9]+)$"));
+    int pos = fontRx.indexIn(font, familyIdx);
 
-    bool italic = !fontRx.cap(1).isEmpty();
-    QFont::Weight bold = fontRx.cap(2).isEmpty() ? QFont::Normal : QFont::Bold;
-    int fontSize = fontRx.cap(3).toInt();
+    QString fontStyle = fontRx.cap(1).trimmed();
+    int fontSize = fontRx.cap(2).toInt();
     
-    return QFont(fontFamily, fontSize, bold, italic);
+    return fdb.font(fontFamily, fontStyle, fontSize);
 }
 
 void GTKConfigKCModule::refreshLists()
@@ -194,6 +206,8 @@ void GTKConfigKCModule::refreshLists()
     
     ui->checkBox_icon_gtk_buttons->setChecked(appareance->getShowIconsInButtons());
     ui->checkBox_icon_gtk_menus->setChecked(appareance->getShowIconsInMenus());
+    
+    makePreviewIconTheme();
 }
 
 void tryIcon(QLabel* label, const QString& fallback, const QString& theme, const QString& iconName)
@@ -326,7 +340,7 @@ void GTKConfigKCModule::save()
             << "********************************************************";
     syncUI();
     if(!appareance->saveFileConfig())
-        QMessageBox::warning(this, "ERROR", i18n("It was not possible to save the config"));
+        KMessageBox::error(this, i18n("It was not possible to save the config"));
 }
 
 void setComboItem(QComboBox* combo, const QStringList& texts)
@@ -353,7 +367,7 @@ void GTKConfigKCModule::defaults()
     setComboItem(ui->cb_theme_gtk3, QStringList("oxygen-gtk") << "Adwaita");
     
     QStringList icons;
-    icons << KIconTheme(KIconTheme::current()).name() << "GNOME";
+    icons << KIconLoader::global()->theme()->name() << "GNOME";
     setComboItem(ui->cb_icon, icons);
     
     int idx = ui->cb_icon->currentIndex();
@@ -369,12 +383,9 @@ void GTKConfigKCModule::defaults()
 void GTKConfigKCModule::load()
 {
     m_saveEnabled = false;
-    refreshThemesUi();
-    
     bool someCorrect = appareance->loadFileConfig();
     if(someCorrect) {
         refreshLists();
-        makePreviewIconTheme();
     } else
         defaults();
     
@@ -436,7 +447,7 @@ void GTKConfigKCModule::showDialogForUninstall()
         connect(uninstaller, SIGNAL(themeUninstalled()), SLOT(refreshLists()));
     }
     
-    uninstaller->refresthListsForUninstall();
+    uninstaller->refreshListsForUninstall();
     uninstaller->exec();
     
     refreshThemesUi();
